/* code03e8.c - EvtProcessSoftKeyStroke trap handler
	Copyright  2000 Lee.Sang-Min <luxmea@chollian.net>
	Distributed under the GNU GPL License
	
	Based on PhlegmHack by Mitch Blevins <mblevin@debian.org>
	which was...
	Base on LinkHistoryHack by Ben Darnell <bgdarnel@unity.ncsu.edu>
	which was...
	Based on SampleHack by Roger Chaplin <foursquaredev@att.net>
*/

#include <Pilot.h>
#include <FeatureMgr.h>
#include <SysEvtPrv.h>
#include <Hardware.h>
#include "eTask.h"
#include "eTaskRsc.h"
#include "callback.h"
#include "DAstack.h"

static Boolean isInLaunchButton(PointType*);
static Boolean isInMenuButton(PointType*);
static Boolean isInULGraffitiArea(PointType*);
static void DoAction(Boolean bShowList);
static void DrawList(UInt, RectanglePtr, TaskItem*);
static void DrawListWithIcon(UInt, RectanglePtr, TaskItem*);

/* This is the code that will be called whenever ANY app calls EvtProcessSoftKeyStroke, 
	and eTask is enabled in HackMaster. HackMaster is not smart about the
	content of this file; it will simply jump to the first executable statement
	in this file. If you need helper functions, prototype them above, and
	define them AFTER the trap patch.
*/
Err MyEvtProcessSoftKeyStroke(PointType* ptStart, PointType* ptEnd)
{
	Err (*oldTrap)(PointType*, PointType*);
	DWord addrValue;

	// Get the address of the old EvtProcessSoftKeyStroke() from HackMaster
	FtrGet(CreatorID, 0x3e8, &addrValue);
	// Set the function pointer to the old trap
	oldTrap = (Err(*)(PointType*, PointType*))addrValue;

	if (isInLaunchButton(ptStart) && isInULGraffitiArea(ptEnd)) {
		DoAction(false);
		return 0;
	}
	else if (isInMenuButton(ptStart) && isInLaunchButton(ptEnd)) {
		// Only if task_list is not already up...
		if (FtrGet(CreatorID, FTR_DUPLICATE, &addrValue) != 0) {
			DoAction(true);
			return 0;
		}
	}
/*	else if (isInFindButton(ptStart)) {
		if (isInLRGraffitiArea(ptEnd))
			return launchDA('DAMe');		// daMemoPad
		else if (isInCalcButton(ptEnd))
			return launchDA('DADB');		// ApptViewerDA
	}
	else if (isInCalcButton(ptStart)) {
		if (isInLRScreenArea(ptEnd))
			return launchDA('eCRT');		// eCurrentDA
		else if (isInFindButton(ptEnd))
			return launchDA('HxDA');		// HexDA
	}
*/	
	return oldTrap(ptStart, ptEnd);
}

static void DrawList(UInt itemIndex, RectanglePtr bounds, TaskItem *taskList)
{	
	CALLBACK_PROLOGUE

	WinEraseRectangle(bounds, 0);
	
	WinDrawChars(taskList[itemIndex].name, StrLen(taskList[itemIndex].name), bounds->topLeft.x+1, bounds->topLeft.y);

	CALLBACK_EPILOGUE
}

static void DrawListWithIcon(UInt itemIndex, RectanglePtr bounds, TaskItem *taskList)
{	
	CALLBACK_PROLOGUE

	WinEraseRectangle(bounds, 0);
	
	WinDrawBitmap((BitmapPtr)(taskList[itemIndex].icon), bounds->topLeft.x, bounds->topLeft.y+1);
	WinDrawChars(taskList[itemIndex].name, StrLen(taskList[itemIndex].name), bounds->topLeft.x+17, bounds->topLeft.y);

	CALLBACK_EPILOGUE
}

static void DoAction(Boolean bShowList)
{
	ListType lstHistory;
	Word prefsSize;
	short selected, offset;
	UInt cardNo, i, nTotalItems;
	LocalID dbID;
	DWord currCreator;
	ListStats listStats;
	TaskList taskList = NULL;
	DmSearchStateType searchState;
	SWord swResult;
	Err errFound;

//	CALLBACK_PROLOGUE
	
	prefsSize = sizeof(ListStats);
	swResult = PrefGetAppPreferences(CreatorID, idListStats, &listStats, &prefsSize, true);
	
	if ((swResult==noPreferenceFound) || (prefsSize!=sizeof(ListStats))) {
		SndPlaySystemSound(sndWarning);
		return;
	}
	
	nTotalItems = listStats.nExecuted+listStats.nFavorite+listStats.nIgnored;
	
	// Get the current task_list
	taskList = MemPtrNew(prefsSize=nTotalItems*sizeof(TaskItem));
	PrefGetAppPreferences(CreatorID, idTaskList, taskList, &prefsSize, true);

	SysCurAppDatabase(&cardNo, &dbID);
	DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &currCreator);
	
	// Decide which action to take...
	if (bShowList == true) {
		// top item = most recently executed application
		switch (currCreator) {
			case 'lnch':
			case 'LcPd':
			case 'SSle':
			case 'ApMn':
			case 'MjBb':
			case 'Cmdr':
			case 'PApL':
			case 'QkLn':
			case 'Lnch':
			case 'CoLn':
			case 'StBr':
				if (listStats.nExecuted > 0) {
					offset = 1;
					break;
				}

			default:
				offset = 0;
		}
		
		for (; offset<listStats.nExecuted; offset++) {
			if ((taskList[offset].creator!=currCreator) && (taskList[offset].flag==IS_REFERENCED))
				break;
		}

		// Set the height and width of list
		MemSet(&lstHistory, sizeof(ListType), 0);
		
		lstHistory.numItems = listStats.nExecuted+listStats.nFavorite-offset;
		
		if (lstHistory.numItems <= 0) {
			SndPlaySystemSound(sndWarning);
			
			if (taskList != NULL) MemPtrFree(taskList);
			return;
		}

		for (i=offset; i<listStats.nExecuted+listStats.nFavorite; i++) {
			if (taskList[i].width > lstHistory.bounds.extent.x)
				lstHistory.bounds.extent.x = taskList[i].width;
		}
		
		if (listStats.showIcon == 0) {
			lstHistory.bounds.extent.x += 5;
			lstHistory.drawItemsCallback = (ListDrawDataFuncPtr)DrawList;
		}
		else {
			lstHistory.bounds.extent.x += 22;	// padding the space for icon
			lstHistory.drawItemsCallback = (ListDrawDataFuncPtr)DrawListWithIcon;
		}

		lstHistory.bounds.extent.y = lstHistory.numItems*FntCharHeight();
		lstHistory.bounds.topLeft.x = 1;
		lstHistory.bounds.topLeft.y = 159-lstHistory.bounds.extent.y;
		lstHistory.attr.usable = true;
		lstHistory.font = stdFont;
		lstHistory.topItem = 0;
		lstHistory.itemsText = (CharPtr*)(taskList+offset);	// This is cheating

		LstSetSelection(&lstHistory, -1);

		// Popup the list
		FtrSet(CreatorID, FTR_DUPLICATE, 0);	// just a warning...
		
		// Click to confirm to the user
		SndPlaySystemSound(sndClick);
		selected = LstPopupList(&lstHistory);
		
		FtrUnregister(CreatorID, FTR_DUPLICATE);	// Okay, safe now.
		
		if (selected < 0) {
//			SndPlaySystemSound(sndWarning);
			if (taskList != NULL) MemPtrFree(taskList);
			return;
		}
	}
	else {
		// Search the most recently executed application
		for (offset=0; offset<listStats.nExecuted; offset++) {
			if (taskList[offset].creator != currCreator)
				break;
		}

		selected = 0;
		
		if (selected+offset >= listStats.nExecuted) {
			SndPlaySystemSound(sndWarning);

			if (taskList != NULL) MemPtrFree(taskList);
			return;
		}
	}

	errFound = DmGetNextDatabaseByTypeCreator(true, &searchState, sysFileTApplication, taskList[selected+offset].creator, true, &cardNo, &dbID);
	if (errFound != 0) {
		DmOpenRef dbRef = DmOpenDatabaseByTypeCreator('HACK', CreatorID, dmModeReadOnly);
		FrmCustomAlert(altDeleted, taskList[selected+offset].name, NULL, NULL);
		DmCloseDatabase(dbRef);
		
//		SndPlaySystemSound(sndWarning);

		if (selected+offset < listStats.nExecuted)
			listStats.nExecuted--;
		else
			listStats.nFavorite--;
		
		nTotalItems--;

		// Remove the offending entry
		MemMove(taskList+selected+offset, taskList+selected+offset+1, (nTotalItems-selected-offset)*sizeof(TaskItem));
		
		PrefSetAppPreferences(CreatorID, idTaskList, 0, taskList, nTotalItems*sizeof(TaskItem), true);
		PrefSetAppPreferences(CreatorID, idListStats, 0, &listStats, sizeof(ListStats), true);
		
		if (taskList != NULL) MemPtrFree(taskList);
	}
	else {
		if (taskList != NULL) MemPtrFree(taskList);
		
		SysUIAppSwitch(cardNo, dbID, sysAppLaunchCmdNormalLaunch, NULL);
	}
/*
cleanUp:
	if (taskList != NULL) MemPtrFree(taskList);
*/
//	CALLBACK_EPILOGUE
}

static Boolean isInLaunchButton(PointType* pt)
{
	if ((pt->x >= hwrLaunchBtnLeft) && (pt->x < hwrLaunchBtnLeft+hwrLaunchBtnWidth))
		if ((pt->y >= hwrLaunchBtnTop) && (pt->y < hwrLaunchBtnTop+hwrLaunchBtnHeight))
			return true;
	return false;
}

static Boolean isInMenuButton(PointType* pt)
{
	if ((pt->x >= hwrMenuBtnLeft) && (pt->x < hwrMenuBtnLeft+hwrMenuBtnWidth))
		if ((pt->y >= hwrMenuBtnTop) && (pt->y < hwrMenuBtnTop+hwrMenuBtnHeight))
			return true;
	return false;
}

static Boolean isInULGraffitiArea(PointType* pt)
{
	if ((pt->x >= hwrGraffitiLeft) && (pt->x < hwrGraffitiSplit))
		if ((pt->y >= hwrGraffitiTop) && (pt->y < hwrGraffitiTop+hwrGraffitiHeight/2))
			return true;
	return false;
}
