/* code07d0.c - Preferences panel
	Copyright  2000 Lee.Sang-Min <luxmea@chollian.net>
	Distributed under the GNU GPL License
	
	Based on PhlegmHack by Mitch Blevins <mblevin@debian.org>
	which was...
	Based on LinkHistoryHack by Ben Darnel <bgdarnel@unity.ncsu.edu>
	which was...
	Based on SampleHack by Roger Chaplin <foursquaredev@att.net>
*/

#include <Pilot.h>
#include "eTask.h"
#include "eTaskRsc.h"
#include "callback.h"

static void PrefsInit(FormPtr, ListPtr, ControlPtr);
static void PrefsOK(FormPtr, ControlPtr);
static void PrefsListDrawFunc(UInt, RectanglePtr, CharPtr*);
static void DoListSelect(ListPtr, short);

/* This is the code that will be called whenever the user taps myhack's '+'
	button in the HackMaster pList. HackMaster is not smart about the content of
	this file; it will simply jump to the first executable statement in this
	file. If you need helper functions, prototype them above, and define them
	AFTER the trap patch.
*/
Boolean PrefsHandler(EventPtr pEvent)
{
	static FormPtr pForm = NULL;
	static ListPtr pList = NULL;
	static ControlPtr pControl = NULL;
	Boolean bHandled = false;

	switch (pEvent->eType) {
		case frmOpenEvent:
			pForm = FrmGetActiveForm();
			pList = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, lstApplication));
			pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, chkShowIcon));
			
			PrefsInit(pForm, pList, pControl);
			FrmDrawForm(pForm);
			
			bHandled = true;
			break;
	
		case lstSelectEvent:
			DoListSelect(pList, pEvent->data.lstSelect.selection);
			break;

		case ctlSelectEvent:
			switch (pEvent->data.ctlEnter.controlID) {
				case chkShowIcon:
					break;
					
				case cmdOK:
					PrefsOK(pForm, pControl);
					// fall through
					
				case cmdCancel:
					FrmGotoForm(9000);	// return to HackMaster's control panel
					bHandled = true;
			}
			break;

		case keyDownEvent:
			switch (pEvent->data.keyDown.chr) {
				case pageUpChr:
					LstScrollList(pList, up, 7);	// visible items = 8
					break;
					
				case pageDownChr:
					LstScrollList(pList, down, 7);
			}
			break;
		
		default:
			break;
	}
	
	return bHandled;
}

/* The control panel event handler cannot rely on global variables.
	Feature manager calls possibly in combination with database or dynamic heap allocation 
	should be used to save whatever state information is necessary between event handler calls.
*/
static void PrefsInit(FormPtr pForm, ListPtr pList, ControlPtr pControl)
{
	Handle hAppList;
	Word dwAppCount;
	SysDBListItemType *pAppList;
	ListStats listStats;
	TaskList taskList = NULL;
	Word prefsSize;
	SWord swResult;
	UInt i, j, nTotalItems;

	// Get every stinking little application
	if (SysCreateDataBaseList('appl', 0, &dwAppCount, &hAppList, true) != 0) {
		// Publish the list & number of applications installed on a device
		FtrSet(CreatorID, FTR_HAPPLIST, (DWord)hAppList);
		FtrSet(CreatorID, FTR_APPCOUNT, (DWord)dwAppCount);
		
		pAppList = MemHandleLock((VoidHand)hAppList);
		
		prefsSize = sizeof(ListStats);
		swResult = PrefGetAppPreferences(CreatorID, idListStats, &listStats, &prefsSize, true);
		
		if ((swResult!=noPreferenceFound) && (prefsSize==sizeof(ListStats))) {
			CtlSetValue(pControl, listStats.showIcon);
			
			nTotalItems = listStats.nExecuted+listStats.nFavorite+listStats.nIgnored;
			
			taskList = MemPtrNew(prefsSize = nTotalItems*sizeof(TaskItem));
			swResult = PrefGetAppPreferences(CreatorID, idTaskList, taskList, &prefsSize, true);
			
			if ((swResult==noPreferenceFound) || (prefsSize!=nTotalItems*sizeof(TaskItem))) {
				nTotalItems = listStats.nExecuted = 0;
			}
		}
		else {
			nTotalItems = listStats.nExecuted = 0;
		}
	
		// Cheat and use version to signify the application which is checked
		for (i=0; i<dwAppCount; i++) {
			switch (pAppList[i].creator) {
				case 'lnch':
				case 'LcPd':
				case 'SSle':
				case 'ApMn':
				case 'MjBb':
				case 'Cmdr':
				case 'PApL':
				case 'QkLn':
				case 'Lnch':
				case 'CoLn':
				case 'StBr':
				case 'date':
				case 'addr':
				case 'todo':
				case 'memo':
				case 'sync':
					pAppList[i].version = IS_IGNORED;
					break;
					
				default:
					pAppList[i].version = IS_RELEASED;
			}
		}
			
		for (i=listStats.nExecuted; i<nTotalItems; i++) {
			for (j=0; j<dwAppCount; j++) {
				if (pAppList[j].creator == taskList[i].creator) {
					pAppList[j].version = taskList[i].flag;
					break;
				}
			}
		}
		
		MemHandleUnlock((VoidHand)hAppList);

		LstSetListChoices(pList, NULL, dwAppCount);
		LstSetDrawFunction(pList, PrefsListDrawFunc);
		LstSetSelection(pList, -1);
	}
	else {	// Why?
		FtrUnregister(CreatorID, FTR_HAPPLIST);
		FtrUnregister(CreatorID, FTR_APPCOUNT);
	}

	// Cleanup
	if (taskList != NULL) MemPtrFree(taskList);
}

static void PrefsListDrawFunc(UInt index, RectanglePtr bounds, CharPtr* data)
{
	Handle hAppList;
	SysDBListItemType *pAppList;
	VoidHand hSymbol;
	Int resID = 0;

	CALLBACK_PROLOGUE

	// Get the list of applications installed on a device
	if (FtrGet(CreatorID, FTR_HAPPLIST, (DWordPtr)&hAppList) != 0) return;
	
	pAppList = MemHandleLock((VoidHand)hAppList);

	switch (pAppList[index].version) {
		case IS_FAVORITE:
			resID = btmFavorite;
			break;
		case IS_IGNORED:
			resID = btmIgnore;
	}
	
	if (resID != 0) {	// Is checked?
		hSymbol = DmGetResource('Tbmp', resID);
		WinDrawBitmap((BitmapPtr)MemHandleLock(hSymbol), bounds->topLeft.x, bounds->topLeft.y+1);
		
		MemHandleUnlock(hSymbol);
		DmReleaseResource(hSymbol);
	}
	
	// Draw the application name
	WinDrawChars(pAppList[index].name, StrLen(pAppList[index].name), bounds->topLeft.x+16, bounds->topLeft.y);

	// Cleanup
	MemHandleUnlock((VoidHand)hAppList);
	
	CALLBACK_EPILOGUE
}

static void DoListSelect(ListPtr pList, short selection)
{
	SysDBListItemType *pAppList;
	Handle hAppList;

	if (selection < 0) return;
	if (FtrGet(CreatorID, FTR_HAPPLIST, (DWordPtr)&hAppList) != 0) return;

	pAppList = MemHandleLock((VoidHand)hAppList);
	
	switch (pAppList[selection].version) {
		case IS_RELEASED:
			pAppList[selection].version = IS_IGNORED;
			break;

		case IS_IGNORED:
			pAppList[selection].version = IS_FAVORITE;
			break;
			
		case IS_FAVORITE:
			pAppList[selection].version = IS_RELEASED;
	}
	
	MemHandleUnlock((VoidHand)hAppList);
	
	// Update the list of applications immediately
	LstEraseList(pList);
	LstSetSelection(pList, -1);
	LstDrawList(pList);
}

static void PrefsOK(FormPtr pForm, ControlPtr pControl)
{
	Handle hAppList;
	DWord dwAppCount;
	SysDBListItemType *pAppList;
	ListStats listStats;
	TaskItem *taskList;
	DmOpenRef dbRef;
	Int resIndex;
	VoidHand hIcon;
	UInt i, j, k, nTotalItems;

	// Get the list & number of applications installed on a device
	if (FtrGet(CreatorID, FTR_HAPPLIST, (DWordPtr)&hAppList) != 0) return;
	if (FtrGet(CreatorID, FTR_APPCOUNT, &dwAppCount) != 0) return;
	
	pAppList = MemHandleLock((VoidHand)hAppList);
	
	listStats.nFavorite = 0;
	listStats.nIgnored = 0;

	// Count the number of applicatons checked
	for (i=0; i<dwAppCount; i++) {
		switch (pAppList[i].version) {
			case IS_FAVORITE:
				listStats.nFavorite++;
				break;

			case IS_IGNORED:
				listStats.nIgnored++;
		}
	}
	
	// Clear the current task_list
	nTotalItems = (listStats.nExecuted=0)+listStats.nFavorite+listStats.nIgnored;
	listStats.showIcon = CtlGetValue(pControl);
	
	taskList = MemPtrNew(nTotalItems*sizeof(TaskItem));

	// index of first favorite application in task_list
	j = listStats.nExecuted;
	// index of first ignored application in task_list
	k = listStats.nExecuted+listStats.nFavorite;
	
	// Collect the details of the applcations which is checked
	for (i=0; i<dwAppCount; i++) {
		switch (pAppList[i].version) {
			case IS_FAVORITE:
			{
				taskList[j].creator = pAppList[i].creator;
				taskList[j].flag = IS_FAVORITE;

				StrCopy(taskList[j].name, pAppList[i].name);
				taskList[j].width = FntCharsWidth(pAppList[i].name, StrLen(pAppList[i].name));
				
				// Get the application's small icon image
				dbRef = DmOpenDatabase(pAppList[i].cardNo, pAppList[i].dbID, dmModeReadOnly);
				
				ErrFatalDisplayIf(dbRef==0, "Could not open the application resource database.");
				
				resIndex = DmFindResource(dbRef, 'tAIB', 1001, NULL);
				if (resIndex != -1) {
					hIcon = DmGetResourceIndex(dbRef, resIndex);
				}
				else {	// if no samll icon
					hIcon = DmGetResource('Tbmp', btmDefault);	// default image
				}
				
				MemMove(taskList[j].icon, (BitmapPtr)MemHandleLock(hIcon), ICON_SIZE);
				
				MemHandleUnlock(hIcon);
				DmReleaseResource(hIcon);
				
				DmCloseDatabase(dbRef);

				j++;
				break;
			}

			case IS_IGNORED:
				taskList[k].creator = pAppList[i].creator;
				taskList[k].flag = IS_IGNORED;
				
				k++;
				break;
		}
	}
	
	MemHandleUnlock((VoidHand)hAppList);
	
	PrefSetAppPreferences(CreatorID, idTaskList, 0, taskList, nTotalItems*sizeof(TaskItem), true);
	PrefSetAppPreferences(CreatorID, idListStats, 0, &listStats, sizeof(ListStats), true);
	
	// Cleanup
	if (taskList != NULL) MemPtrFree(taskList);
}
